@@ -44,6 +44,26 @@
 #include <QList>
 #include <QStringList>
 
+struct LibArchiveInterface::ArchiveReadCustomDeleter
+{
+    static inline void cleanup(struct archive *a)
+    {
+        if (a) {
+            archive_read_finish(a);
+        }
+    }
+};
+
+struct LibArchiveInterface::ArchiveWriteCustomDeleter
+{
+    static inline void cleanup(struct archive *a)
+    {
+        if (a) {
+            archive_write_finish(a);
+        }
+    }
+};
+
 LibArchiveInterface::LibArchiveInterface(QObject *parent, const QVariantList & args)
         : ReadWriteArchiveInterface(parent, args),
         m_cachedArchiveEntryCount(0),
@@ -60,57 +80,52 @@ LibArchiveInterface::~LibArchiveInterface()
 bool LibArchiveInterface::list()
 {
     kDebug();
-    struct archive *arch;
-    struct archive_entry *aentry;
-    int result;
 
-    arch = archive_read_new();
-    if (!arch) {
+    ArchiveRead arch_reader(archive_read_new());
+
+    if (!(arch_reader.data())) {
         return false;
     }
 
-    result = archive_read_support_compression_all(arch);
-    if (result != ARCHIVE_OK) {
-        archive_read_finish(arch);
+    if (archive_read_support_compression_all(arch_reader.data()) != ARCHIVE_OK) {
         return false;
     }
 
-    result = archive_read_support_format_all(arch);
-    if (result != ARCHIVE_OK) {
-        archive_read_finish(arch);
+    if (archive_read_support_format_all(arch_reader.data()) != ARCHIVE_OK) {
         return false;
     }
 
-    result = archive_read_open_filename(arch, QFile::encodeName(filename()), 10240);
-
-    if (result != ARCHIVE_OK) {
-        error(i18n("Could not open the file '%1', libarchive cannot handle it.", filename()), QString());
-        archive_read_finish(arch);
+    if (archive_read_open_filename(arch_reader.data(), QFile::encodeName(filename()), 10240) != ARCHIVE_OK) {
+        error(i18n("Could not open the file '%1', libarchive cannot handle it.",
+                   filename()), QString());
         return false;
     }
 
     m_cachedArchiveEntryCount = 0;
     m_extractedFilesSize = 0;
 
-    while ((result = archive_read_next_header(arch, &aentry)) == ARCHIVE_OK) {
+    struct archive_entry *aentry;
+    int result;
+
+    while ((result = archive_read_next_header(arch_reader.data(), &aentry)) == ARCHIVE_OK) {
         if (!m_emitNoEntries) {
             emitEntryFromArchiveEntry(aentry);
         }
 
-        m_extractedFilesSize += (qlonglong) archive_entry_size(aentry);
+        m_extractedFilesSize += (qlonglong)archive_entry_size(aentry);
 
         m_cachedArchiveEntryCount++;
-        archive_read_data_skip(arch);
+        archive_read_data_skip(arch_reader.data());
     }
 
     if (result != ARCHIVE_EOF) {
-        error(i18n("The archive reading failed with message: %1", archive_error_string(arch)));
-        archive_read_finish(arch);
+        error(i18n("The archive reading failed with message: %1",
+                   archive_error_string(arch_reader.data())));
         return false;
     }
 
 #if (ARCHIVE_API_VERSION>1)
-    return archive_read_finish(arch) == ARCHIVE_OK;
+    return archive_read_close(arch_reader.data()) == ARCHIVE_OK;
 #else
     return true;
 #endif
@@ -124,39 +139,39 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
     const bool extractAll = files.isEmpty();
     const bool preservePaths = options.value("PreservePaths").toBool();
 
-    struct archive *arch, *writer;
-    struct archive_entry *entry;
+    const QString rootNode = options.value("RootNode", QVariant()).toString();
+    kDebug() << "Set root node" << rootNode;
 
-    QString rootNode;
-    if (options.contains("RootNode")) {
-        rootNode = options.value("RootNode").toString();
-        kDebug() << "Set root node " << rootNode;
-    }
+    ArchiveRead arch(archive_read_new());
 
-    arch = archive_read_new();
-    if (!arch) {
+    if (!(arch.data())) {
         return false;
     }
 
-    archive_read_support_compression_all(arch);
-    archive_read_support_format_all(arch);
+    if (archive_read_support_compression_all(arch.data()) != ARCHIVE_OK) {
+        return false;
+    }
 
-    if (archive_read_open_filename(arch, QFile::encodeName(filename()), 10240) != ARCHIVE_OK) {
+    if (archive_read_support_format_all(arch.data()) != ARCHIVE_OK) {
+        return false;
+    }
+
+    if (archive_read_open_filename(arch.data(), QFile::encodeName(filename()), 10240) != ARCHIVE_OK) {
         error(i18n("Unable to open the file '%1', libarchive cannot handle it.",
                    filename()));
-        archive_read_finish(arch);
         return false;
     }
 
-    writer = archive_write_disk_new();
-    if (!writer) {
-        archive_read_finish(arch);
+    ArchiveWrite writer(archive_write_disk_new());
+    if (!(writer.data())) {
         return false;
     }
 
-    archive_write_disk_set_options(writer, extractionFlags());
+    archive_write_disk_set_options(writer.data(), extractionFlags());
+
+    int entryNr = 0;
+    int totalCount = 0;
 
-    int entryNr = 0, totalCount = 0;
     if (extractAll) {
         if (!m_cachedArchiveEntryCount) {
             progress(0);
@@ -176,8 +191,9 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
 
     bool overwriteAll = false; // Whether to overwrite all files
     bool skipAll = false; // Whether to skip all files
+    struct archive_entry *entry;
 
-    while (archive_read_next_header(arch, &entry) == ARCHIVE_OK) {
+    while (archive_read_next_header(arch.data(), &entry) == ARCHIVE_OK) {
         // retry with renamed entry, fire an overwrite query again
         // if the new entry also exists
     retry:
@@ -185,7 +201,7 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
 
         //we skip directories if not preserving paths
         if (!preservePaths && entryIsDir) {
-            archive_read_data_skip(arch);
+            archive_read_data_skip(arch.data());
             continue;
         }
 
@@ -197,9 +213,6 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
             //TODO: find out what to do here!!
             error(i18n("This archive contains archive entries with absolute paths, which are not yet supported by ark."));
 
-            archive_read_finish(arch);
-            archive_write_finish(arch);
-
             return false;
         }
 
@@ -235,7 +248,7 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
             //now check if the file about to be written already exists
             if (!entryIsDir && entryFI.exists()) {
                 if (skipAll) {
-                    archive_read_data_skip(arch);
+                    archive_read_data_skip(arch.data());
                     archive_entry_clear(entry);
                     continue;
                 } else if (!overwriteAll && !skipAll) {
@@ -244,15 +257,15 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
                     query.waitForResponse();
 
                     if (query.responseCancelled()) {
-                        archive_read_data_skip(arch);
+                        archive_read_data_skip(arch.data());
                         archive_entry_clear(entry);
                         break;
                     } else if (query.responseSkip()) {
-                        archive_read_data_skip(arch);
+                        archive_read_data_skip(arch.data());
                         archive_entry_clear(entry);
                         continue;
                     } else if (query.responseAutoSkip()) {
-                        archive_read_data_skip(arch);
+                        archive_read_data_skip(arch.data());
                         archive_entry_clear(entry);
                         skipAll = true;
                         continue;
@@ -273,17 +286,17 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
                 } else {
                     kDebug(1601) << "Warning, existing, but non-writable dir. skipping";
                     archive_entry_clear(entry);
-                    archive_read_data_skip(arch);
+                    archive_read_data_skip(arch.data());
                     continue;
                 }
             }
 
             int header_response;
             kDebug() << "Writing " << fileWithoutPath << " to " << archive_entry_pathname(entry);
-            if ((header_response = archive_write_header(writer, entry)) == ARCHIVE_OK)
+            if ((header_response = archive_write_header(writer.data(), entry)) == ARCHIVE_OK)
                 //if the whole archive is extracted and the total filesize is
                 //available, we use partial progress
-                copyData(arch, writer, (extractAll && m_extractedFilesSize));
+                copyData(arch.data(), writer.data(), (extractAll && m_extractedFilesSize));
             else if (header_response == ARCHIVE_WARN) {
                 kDebug() << "Warning while writing " << entryName;
             } else {
@@ -300,14 +313,12 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
             }
             archive_entry_clear(entry);
         } else {
-            archive_read_data_skip(arch);
+            archive_read_data_skip(arch.data());
         }
     }
 
-    archive_write_finish(writer);
-
 #if (ARCHIVE_API_VERSION>1)
-    return archive_read_finish(arch) == ARCHIVE_OK;
+    return archive_read_close(arch.data()) == ARCHIVE_OK;
 #else
     return true;
 #endif
@@ -315,15 +326,10 @@ bool LibArchiveInterface::copyFiles(const QList<QVariant> & files, const QString
 
 bool LibArchiveInterface::addFiles(const QStringList & files, const CompressionOptions& options)
 {
-    struct archive *arch_reader = NULL, *arch_writer = NULL;
-    struct archive_entry *entry;
-    int header_response;
-    int ret;
     const bool creatingNewFile = !QFileInfo(filename()).exists();
-
     const QString tempFilename = filename() + ".arkWriting";
-
     const QString globalWorkDir = options.value("GlobalWorkDir").toString();
+
     if (!globalWorkDir.isEmpty()) {
         kDebug() << "GlobalWorkDir is set, changing dir to " << globalWorkDir;
         m_workDir.setPath(globalWorkDir);
@@ -332,130 +338,114 @@ bool LibArchiveInterface::addFiles(const QStringList & files, const CompressionO
 
     m_writtenFiles.clear();
 
+    ArchiveRead arch_reader;
     if (!creatingNewFile) {
-        //*********initialize the reader
-        arch_reader = archive_read_new();
-        if (!arch_reader) {
+        arch_reader.reset(archive_read_new());
+        if (!(arch_reader.data())) {
             error(i18n("The archive reader could not be initialized."));
             return false;
         }
 
-        archive_read_support_compression_all(arch_reader);
-        archive_read_support_format_all(arch_reader);
-        ret = archive_read_open_filename(arch_reader, QFile::encodeName(filename()), 10240);
-        if (ret != ARCHIVE_OK) {
+        if (archive_read_support_compression_all(arch_reader.data()) != ARCHIVE_OK) {
+            return false;
+        }
+
+        if (archive_read_support_format_all(arch_reader.data()) != ARCHIVE_OK) {
+            return false;
+        }
+
+        if (archive_read_open_filename(arch_reader.data(), QFile::encodeName(filename()), 10240) != ARCHIVE_OK) {
             error(i18n("The source file could not be read."));
-            archive_read_finish(arch_reader);
             return false;
         }
     }
 
-    //*********initialize the writer
-    arch_writer = archive_write_new();
-    if (!arch_writer) {
+    ArchiveWrite arch_writer(archive_write_new());
+    if (!(arch_writer.data())) {
         error(i18n("The archive writer could not be initialized."));
         return false;
     }
 
     //pax_restricted is the libarchive default, let's go with that.
-    archive_write_set_format_pax_restricted(arch_writer);
+    archive_write_set_format_pax_restricted(arch_writer.data());
 
+    int ret;
     if (creatingNewFile) {
         if (filename().right(2).toUpper() == "GZ") {
             kDebug() << "Detected gzip compression for new file";
-            ret = archive_write_set_compression_gzip(arch_writer);
+            ret = archive_write_set_compression_gzip(arch_writer.data());
         } else if (filename().right(3).toUpper() == "BZ2") {
             kDebug() << "Detected bzip2 compression for new file";
-            ret = archive_write_set_compression_bzip2(arch_writer);
+            ret = archive_write_set_compression_bzip2(arch_writer.data());
 #ifdef HAVE_LIBARCHIVE_XZ_SUPPORT
         } else if (filename().right(2).toUpper() == "XZ") {
             kDebug() << "Detected xz compression for new file";
-            ret = archive_write_set_compression_xz(arch_writer);
+            ret = archive_write_set_compression_xz(arch_writer.data());
 #endif
 #ifdef HAVE_LIBARCHIVE_LZMA_SUPPORT
         } else if (filename().right(4).toUpper() == "LZMA") {
             kDebug() << "Detected lzma compression for new file";
-            ret = archive_write_set_compression_lzma(arch_writer);
+            ret = archive_write_set_compression_lzma(arch_writer.data());
 #endif
         } else if (filename().right(3).toUpper() == "TAR") {
             kDebug() << "Detected no compression for new file (pure tar)";
-            ret = archive_write_set_compression_none(arch_writer);
+            ret = archive_write_set_compression_none(arch_writer.data());
         } else {
             kDebug() << "Falling back to gzip";
-            ret = archive_write_set_compression_gzip(arch_writer);
+            ret = archive_write_set_compression_gzip(arch_writer.data());
         }
 
         if (ret != ARCHIVE_OK) {
-            error(i18n("Setting compression failed with the error '%1'", QString(archive_error_string(arch_writer))));
-
-            archive_write_finish(arch_writer);
+            error(i18n("Setting compression failed with the error '%1'",
+                       QString(archive_error_string(arch_writer.data()))));
 
             return false;
         }
     } else {
-        switch (archive_compression(arch_reader)) {
+        switch (archive_compression(arch_reader.data())) {
         case ARCHIVE_COMPRESSION_GZIP:
-            ret = archive_write_set_compression_gzip(arch_writer);
+            ret = archive_write_set_compression_gzip(arch_writer.data());
             break;
         case ARCHIVE_COMPRESSION_BZIP2:
-            ret = archive_write_set_compression_bzip2(arch_writer);
+            ret = archive_write_set_compression_bzip2(arch_writer.data());
             break;
 #ifdef HAVE_LIBARCHIVE_XZ_SUPPORT
         case ARCHIVE_COMPRESSION_XZ:
-            ret = archive_write_set_compression_xz(arch_writer);
+            ret = archive_write_set_compression_xz(arch_writer.data());
             break;
 #endif
 #ifdef HAVE_LIBARCHIVE_LZMA_SUPPORT
         case ARCHIVE_COMPRESSION_LZMA:
-            ret = archive_write_set_compression_lzma(arch_writer);
+            ret = archive_write_set_compression_lzma(arch_writer.data());
             break;
 #endif
         case ARCHIVE_COMPRESSION_NONE:
-            ret = archive_write_set_compression_none(arch_writer);
+            ret = archive_write_set_compression_none(arch_writer.data());
             break;
         default:
-            error(i18n("The compression type '%1' is not supported by Ark.", QString(archive_compression_name(arch_reader))));
-
-            archive_read_finish(arch_reader);
-            archive_write_finish(arch_writer);
-
+            error(i18n("The compression type '%1' is not supported by Ark.", QString(archive_compression_name(arch_reader.data()))));
             return false;
         }
 
         if (ret != ARCHIVE_OK) {
-            error(i18n("Setting compression failed with the error '%1'", QString(archive_error_string(arch_writer))));
-
-            archive_read_finish(arch_reader);
-            archive_write_finish(arch_writer);
-
+            error(i18n("Setting compression failed with the error '%1'", QString(archive_error_string(arch_writer.data()))));
             return false;
         }
     }
 
-    ret = archive_write_open_filename(arch_writer, QFile::encodeName(tempFilename));
+    ret = archive_write_open_filename(arch_writer.data(), QFile::encodeName(tempFilename));
     if (ret != ARCHIVE_OK) {
-        error(i18n("Opening the archive for writing failed with error message '%1'", QString(archive_error_string(arch_writer))));
-
-        if (!creatingNewFile) {
-            archive_read_finish(arch_reader);
-        }
-        archive_write_finish(arch_writer);
-
+        error(i18n("Opening the archive for writing failed with error message '%1'", QString(archive_error_string(arch_writer.data()))));
         return false;
     }
 
-    entry = archive_entry_new();
-
     //**************** first write the new files
     foreach(const QString& selectedFile, files) {
         bool success;
 
-        success = writeFile(selectedFile, arch_writer, entry);
+        success = writeFile(selectedFile, arch_writer.data());
 
         if (!success) {
-            archive_read_finish(arch_reader);
-            archive_write_finish(arch_writer);
-
             return false;
         }
 
@@ -467,42 +457,36 @@ bool LibArchiveInterface::addFiles(const QStringList & files, const CompressionO
                 if (it.fileName() == ".." || it.fileName() == ".") continue;
 
                 success = writeFile(path +
-                                    (it.fileInfo().isDir() ? "/" : "")
-                                    , arch_writer, entry);
+                                    (it.fileInfo().isDir() ? "/" : ""),
+                                    arch_writer.data());
 
                 if (!success) {
-                    if (!creatingNewFile) {
-                        archive_read_finish(arch_reader);
-                    }
-                    archive_write_finish(arch_writer);
-
                     return false;
                 }
             }
         }
     }
 
+    struct archive_entry *entry;
+
     //and if we have old elements...
     if (!creatingNewFile) {
         //********** copy old elements from previous archive to new archive
-        while (archive_read_next_header(arch_reader, &entry) == ARCHIVE_OK) {
+        while (archive_read_next_header(arch_reader.data(), &entry) == ARCHIVE_OK) {
             if (m_writtenFiles.contains(QFile::decodeName(archive_entry_pathname(entry)))) {
-                archive_read_data_skip(arch_reader);
+                archive_read_data_skip(arch_reader.data());
                 kDebug() << "Entry already existing, will be refresh: ===> " << archive_entry_pathname(entry);
                 continue;
             }
 
+            int header_response;
             //kDebug() << "Writing entry " << fn;
-            if ((header_response = archive_write_header(arch_writer, entry)) == ARCHIVE_OK)
+            if ((header_response = archive_write_header(arch_writer.data(), entry)) == ARCHIVE_OK)
                 //if the whole archive is extracted and the total filesize is
                 //available, we use partial progress
-                copyData(arch_reader, arch_writer, false);
+                copyData(arch_reader.data(), arch_writer.data(), false);
             else {
                 kDebug() << "Writing header failed with error code " << header_response;
-
-                archive_read_finish(arch_reader);
-                archive_write_finish(arch_writer);
-
                 return false;
             }
 
@@ -510,11 +494,7 @@ bool LibArchiveInterface::addFiles(const QStringList & files, const CompressionO
         }
     }
 
-    ret = archive_write_finish(arch_writer);
-
     if (!creatingNewFile) {
-        archive_read_finish(arch_reader);
-
         //everything seems OK, so we remove the source file and replace it with
         //the new one.
         //TODO: do some extra checks to see if this is really OK
@@ -528,119 +508,97 @@ bool LibArchiveInterface::addFiles(const QStringList & files, const CompressionO
 
 bool LibArchiveInterface::deleteFiles(const QList<QVariant> & files)
 {
-    struct archive *arch_reader = NULL, *arch_writer = NULL;
-    struct archive_entry *entry;
-    int header_response;
-    int ret;
-
     QString tempFilename = filename() + ".arkWriting";
 
-    arch_reader = archive_read_new();
-    if (!arch_reader) {
+    ArchiveRead arch_reader(archive_read_new());
+    if (!(arch_reader.data())) {
         error(i18n("The archive reader could not be initialized."));
         return false;
     }
 
-    archive_read_support_compression_all(arch_reader);
-    archive_read_support_format_all(arch_reader);
-    ret = archive_read_open_filename(arch_reader, QFile::encodeName(filename()), 10240);
-    if (ret != ARCHIVE_OK) {
-        error(i18n("The source file could not be read."));
-        archive_read_finish(arch_reader);
+    if (archive_read_support_compression_all(arch_reader.data()) != ARCHIVE_OK) {
         return false;
     }
 
-    //*********initialize the writer
-    arch_writer = archive_write_new();
-    if (!arch_writer) {
+    if (archive_read_support_format_all(arch_reader.data()) != ARCHIVE_OK) {
+        return false;
+    }
+
+    if (archive_read_open_filename(arch_reader.data(), QFile::encodeName(filename()), 10240) != ARCHIVE_OK) {
+        error(i18n("The source file could not be read."));
+        return false;
+    }
+
+    ArchiveWrite arch_writer(archive_write_new());
+    if (!(arch_writer.data())) {
         error(i18n("The archive writer could not be initialized."));
-        archive_read_finish(arch_reader);
         return false;
     }
 
     //pax_restricted is the libarchive default, let's go with that.
-    archive_write_set_format_pax_restricted(arch_writer);
+    archive_write_set_format_pax_restricted(arch_writer.data());
 
-    switch (archive_compression(arch_reader)) {
+    int ret;
+    switch (archive_compression(arch_reader.data())) {
     case ARCHIVE_COMPRESSION_GZIP:
-        ret = archive_write_set_compression_gzip(arch_writer);
+        ret = archive_write_set_compression_gzip(arch_writer.data());
         break;
     case ARCHIVE_COMPRESSION_BZIP2:
-        ret = archive_write_set_compression_bzip2(arch_writer);
+        ret = archive_write_set_compression_bzip2(arch_writer.data());
         break;
 #ifdef HAVE_LIBARCHIVE_XZ_SUPPORT
     case ARCHIVE_COMPRESSION_XZ:
-        ret = archive_write_set_compression_xz(arch_writer);
+        ret = archive_write_set_compression_xz(arch_writer.data());
         break;
 #endif
 #ifdef HAVE_LIBARCHIVE_LZMA_SUPPORT
     case ARCHIVE_COMPRESSION_LZMA:
-        ret = archive_write_set_compression_lzma(arch_writer);
+        ret = archive_write_set_compression_lzma(arch_writer.data());
         break;
 #endif
     case ARCHIVE_COMPRESSION_NONE:
-        ret = archive_write_set_compression_none(arch_writer);
+        ret = archive_write_set_compression_none(arch_writer.data());
         break;
     default:
-        error(i18n("The compression type '%1' is not supported by Ark.", QString(archive_compression_name(arch_reader))));
-
-        archive_read_finish(arch_reader);
-        archive_write_finish(arch_writer);
-
+        error(i18n("The compression type '%1' is not supported by Ark.", QString(archive_compression_name(arch_reader.data()))));
         return false;
     }
 
     if (ret != ARCHIVE_OK) {
-        error(i18n("Setting compression failed with the error '%1'", QString(archive_error_string(arch_writer))));
-
-        archive_read_finish(arch_reader);
-        archive_write_finish(arch_writer);
-
+        error(i18n("Setting compression failed with the error '%1'", QString(archive_error_string(arch_writer.data()))));
         return false;
     }
 
-    ret = archive_write_open_filename(arch_writer, QFile::encodeName(tempFilename));
+    ret = archive_write_open_filename(arch_writer.data(), QFile::encodeName(tempFilename));
     if (ret != ARCHIVE_OK) {
-        error(i18n("Opening the archive for writing failed with error message '%1'", QString(archive_error_string(arch_writer))));
-
-        archive_read_finish(arch_reader);
-        archive_write_finish(arch_writer);
-
+        error(i18n("Opening the archive for writing failed with error message '%1'", QString(archive_error_string(arch_writer.data()))));
         return false;
     }
 
-    entry = archive_entry_new();
+    struct archive_entry *entry;
 
     //********** copy old elements from previous archive to new archive
-    while (archive_read_next_header(arch_reader, &entry) == ARCHIVE_OK) {
+    while (archive_read_next_header(arch_reader.data(), &entry) == ARCHIVE_OK) {
         if (files.contains(QFile::decodeName(archive_entry_pathname(entry)))) {
-            archive_read_data_skip(arch_reader);
-            kDebug() << "Entry to be deleted, skipping" << archive_entry_pathname(entry);
+            archive_read_data_skip(arch_reader.data());
+            kDebug() << "Entry to be deleted, skipping"
+                     << archive_entry_pathname(entry);
             entryRemoved(QFile::decodeName(archive_entry_pathname(entry)));
             continue;
         }
 
+        int header_response;
         //kDebug() << "Writing entry " << fn;
-        if ((header_response = archive_write_header(arch_writer, entry)) == ARCHIVE_OK)
-
+        if ((header_response = archive_write_header(arch_writer.data(), entry)) == ARCHIVE_OK) {
             //if the whole archive is extracted and the total filesize is
             //available, we use partial progress
-            copyData(arch_reader, arch_writer, false);
-        else {
+            copyData(arch_reader.data(), arch_writer.data(), false);
+        } else {
             kDebug() << "Writing header failed with error code " << header_response;
-
-            archive_read_finish(arch_reader);
-            archive_write_finish(arch_writer);
-
             return false;
         }
-
-        archive_entry_clear(entry);
     }
 
-    archive_write_finish(arch_writer);
-    archive_read_finish(arch_reader);
-
     //everything seems OK, so we remove the source file and replace it with
     //the new one.
     //TODO: do some extra checks to see if this is really OK
@@ -653,25 +611,32 @@ bool LibArchiveInterface::deleteFiles(const QList<QVariant> & files)
 void LibArchiveInterface::emitEntryFromArchiveEntry(struct archive_entry *aentry)
 {
     ArchiveEntry e;
+
 #ifdef _MSC_VER
-    e[ FileName ] = QDir::fromNativeSeparators(QString::fromUtf16((ushort*)archive_entry_pathname_w(aentry)));
+    e[FileName] = QDir::fromNativeSeparators(QString::fromUtf16((ushort*)archive_entry_pathname_w(aentry)));
 #else
-    e[ FileName ] = QDir::fromNativeSeparators(QString::fromWCharArray(archive_entry_pathname_w(aentry)));
+    e[FileName] = QDir::fromNativeSeparators(QString::fromWCharArray(archive_entry_pathname_w(aentry)));
 #endif
-    e[ InternalID ] = e[ FileName ];
+    e[InternalID] = e[FileName];
 
-    QString owner = QString(archive_entry_uname(aentry));
-    if (!owner.isEmpty()) e[ Owner ] = owner;
-
-    QString group = QString(archive_entry_gname(aentry));
-    if (!group.isEmpty()) e[ Group ] = group;
-
-    e[ Size ] = (qlonglong) archive_entry_size(aentry);
-    e[ IsDirectory ] = S_ISDIR(archive_entry_mode(aentry));     // see stat(2)
-    if (archive_entry_symlink(aentry)) {
-        e[ Link ] = archive_entry_symlink(aentry);
+    const QString owner = QString::fromAscii(archive_entry_uname(aentry));
+    if (!owner.isEmpty()) {
+        e[Owner] = owner;
     }
-    e[ Timestamp ] = QDateTime::fromTime_t(archive_entry_mtime(aentry));
+
+    const QString group = QString::fromAscii(archive_entry_gname(aentry));
+    if (!group.isEmpty()) {
+        e[Group] = group;
+    }
+
+    e[Size] = (qlonglong)archive_entry_size(aentry);
+    e[IsDirectory] = S_ISDIR(archive_entry_mode(aentry));
+
+    if (archive_entry_symlink(aentry)) {
+        e[Link] = archive_entry_symlink(aentry);
+    }
+
+    e[Timestamp] = QDateTime::fromTime_t(archive_entry_mtime(aentry));
 
     entry(e);
 }
@@ -747,7 +712,7 @@ void LibArchiveInterface::copyData(struct archive *source, struct archive *dest,
     }
 }
 
-bool LibArchiveInterface::writeFile(const QString& fileName, struct archive* arch_writer, struct archive_entry* entry)
+bool LibArchiveInterface::writeFile(const QString& fileName, struct archive* arch_writer)
 {
     KDE_struct_stat st;
     int header_response;
@@ -760,25 +725,31 @@ bool LibArchiveInterface::writeFile(const QString& fileName, struct archive* arc
     //       class hierarchy to avoid code duplication
     const QString relativeName = m_workDir.relativeFilePath(fileName) + (trailingSlash ? "/" : "");
 
+    struct archive_entry *entry = archive_entry_new();
+
     KDE_stat(QFile::encodeName(relativeName).constData(), &st);
     archive_entry_copy_stat(entry, &st);
     archive_entry_copy_pathname(entry, QFile::encodeName(relativeName).constData());
 
     kDebug() << "Writing new entry " << archive_entry_pathname(entry);
-    if ((header_response = archive_write_header(arch_writer, entry)) == ARCHIVE_OK)
+    if ((header_response = archive_write_header(arch_writer, entry)) == ARCHIVE_OK) {
         //if the whole archive is extracted and the total filesize is
         //available, we use partial progress
         copyData(fileName, arch_writer, false);
-    else {
+    } else {
         kDebug() << "Writing header failed with error code " << header_response;
         kDebug() << "Error while writing..." << archive_error_string(arch_writer) << "(error nb =" << archive_errno(arch_writer) << ')';
+
+        archive_entry_free(entry);
+
         return false;
     }
 
     m_writtenFiles.push_back(relativeName);
 
     emitEntryFromArchiveEntry(entry);
-    archive_entry_clear(entry);
+
+    archive_entry_free(entry);
 
     return true;
 }
