@@ -27,62 +27,62 @@
 #include <kmenu.h>
 
 KCalcConstButton::KCalcConstButton(QWidget *parent)
-  : KCalcButton(parent), _button_num(-1)
+        : KCalcButton(parent), _button_num(-1)
 {
-  addMode(ModeInverse, i18nc("Write display data into memory", "Store"), i18n("Write display data into memory"));
+    addMode(ModeShift, i18nc("Write display data into memory", "Store"), i18n("Write display data into memory"));
 
-  initPopupMenu();
+    initPopupMenu();
 
-  connect(this, SIGNAL(clicked()), SLOT(slotClicked()));
+    connect(this, SIGNAL(clicked()), SLOT(slotClicked()));
 }
 
 
 KCalcConstButton::KCalcConstButton(const QString &label, QWidget *parent,
-				   const QString &tooltip)
-  : KCalcButton(label, parent, tooltip), _button_num(-1)
+                                   const QString &tooltip)
+        : KCalcButton(label, parent, tooltip), _button_num(-1)
 {
-  addMode(ModeInverse, i18nc("Write display data into memory", "Store"), i18n("Write display data into memory"));
+    addMode(ModeShift, i18nc("Write display data into memory", "Store"), i18n("Write display data into memory"));
 
-  initPopupMenu();
+    initPopupMenu();
 }
 
 QString KCalcConstButton::constant(void) const
 {
-  return KCalcSettings::valueConstant(_button_num);
+    return KCalcSettings::valueConstant(_button_num);
 }
 
 void KCalcConstButton::setButtonNumber(int num)
 {
-	_button_num = num;
+    _button_num = num;
 }
 
 void KCalcConstButton::setLabelAndTooltip(void)
 {
-  QString new_label = QString("C") + QString().setNum(_button_num + 1);
-  QString new_tooltip;
+    QString new_label = QString("C") + QString().setNum(_button_num + 1);
+    QString new_tooltip;
 
-  new_label = (KCalcSettings::nameConstant(_button_num).isNull() ? new_label : KCalcSettings::nameConstant(_button_num));
-  
-  new_tooltip = new_label + '=' + KCalcSettings::valueConstant(_button_num);
-  
-  addMode(ModeNormal, new_label, new_tooltip);
+    new_label = (KCalcSettings::nameConstant(_button_num).isNull() ? new_label : KCalcSettings::nameConstant(_button_num));
+
+    new_tooltip = new_label + '=' + KCalcSettings::valueConstant(_button_num);
+
+    addMode(ModeNormal, new_label, new_tooltip);
 }
 
 void KCalcConstButton::initPopupMenu(void)
 {
-  KCalcConstMenu *tmp_menu = new KCalcConstMenu(this);
-  
-  QAction *a = new QAction(this);
-  a->setText(i18n("Set Name"));
-  connect(a, SIGNAL(triggered()), this, SLOT(slotConfigureButton()));
-  addAction(a);
-  tmp_menu->menuAction()->setText(i18n("Choose From List"));
-  addAction(tmp_menu->menuAction());
-  setContextMenuPolicy(Qt::ActionsContextMenu);
-  
-  connect(tmp_menu,
-	  SIGNAL(triggeredConstant(science_constant const &)),
-	  SLOT(slotChooseScientificConst(science_constant const &)));
+    KCalcConstMenu *tmp_menu = new KCalcConstMenu(this);
+
+    QAction *a = new QAction(this);
+    a->setText(i18n("Set Name"));
+    connect(a, SIGNAL(triggered()), this, SLOT(slotConfigureButton()));
+    addAction(a);
+    tmp_menu->menuAction()->setText(i18n("Choose From List"));
+    addAction(tmp_menu->menuAction());
+    setContextMenuPolicy(Qt::ActionsContextMenu);
+
+    connect(tmp_menu,
+            SIGNAL(triggeredConstant(science_constant const &)),
+            SLOT(slotChooseScientificConst(science_constant const &)));
 
 }
 
@@ -90,8 +90,8 @@ void KCalcConstButton::slotConfigureButton()
 {
     bool yes_no;
     QString input = KInputDialog::getText(i18n("New Name for Constant"), i18n("New name:"),
-					 text(), &yes_no, this); // "nameUserConstants-Dialog"
-    if(yes_no) {
+                                          text(), &yes_no, this);  // "nameUserConstants-Dialog"
+    if (yes_no) {
         KCalcSettings::setNameConstant(_button_num, input);
         setLabelAndTooltip();
     }
@@ -99,16 +99,16 @@ void KCalcConstButton::slotConfigureButton()
 
 void KCalcConstButton::slotChooseScientificConst(struct science_constant const &const_chosen)
 {
-  KCalcSettings::setValueConstant(_button_num, const_chosen.value);
+    KCalcSettings::setValueConstant(_button_num, const_chosen.value);
 
-  KCalcSettings::setNameConstant(_button_num,const_chosen.label);
-  
-  setLabelAndTooltip();
+    KCalcSettings::setNameConstant(_button_num, const_chosen.label);
+
+    setLabelAndTooltip();
 }
 
 void KCalcConstButton::slotClicked()
 {
-	emit clicked(_button_num);
+    emit clicked(_button_num);
 }
 
 #include "kcalc_const_button.moc"
