@@ -1,10 +1,7 @@
-/***************************************************************************
-                          keysmanager.cpp  -  description
-                             -------------------
-    begin                : Thu Jul 4 2002
-    copyright          : (C) 2002 by Jean-Baptiste Mardelle
-    email                : bj@altern.org
- ***************************************************************************/
+/*
+ * Copyright (C) 2002 Jean-Baptiste Mardelle <bj@altern.org>
+ * Copyright (C) 2007,2008,2009,2010 Rolf Eike Beer <kde@opensource.sf-tec.de>
+ */
 
 /***************************************************************************
  *                                                                         *
@@ -28,7 +25,6 @@
 #include <QWidget>
 #include <QLabel>
 #include <QEvent>
-#include <QMovie>
 #include <QList>
 #include <QFile>
 #include <QDir>
@@ -71,6 +67,7 @@
 #include <kio/global.h>
 #include <kjobtrackerinterface.h>
 
+#include "caff.h"
 #include "kgpgkey.h"
 #include "selectsecretkey.h"
 #include "newkey.h"
@@ -97,6 +94,7 @@
 #include "kgpgadduid.h"
 #include "kgpgexport.h"
 #include "kgpggeneratekey.h"
+#include "kgpggeneraterevoke.h"
 #include "kgpgdelkey.h"
 #include "kgpgimport.h"
 #include "kgpgprimaryuid.h"
@@ -109,6 +107,8 @@
 #include "kgpgtransactionjob.h"
 #include "kgpgsignkey.h"
 #include "kgpgsignuid.h"
+#include "kgpgdelsign.h"
+#include "kgpgdecrypt.h"
 
 using namespace KgpgCore;
 
@@ -296,6 +296,9 @@ KeysManager::KeysManager(QWidget *parent)
 	signUid = actionCollection()->addAction("key_sign_uid");
 	signUid->setIcon(KIcon("document-sign-key"));
 	connect(signUid, SIGNAL(triggered(bool)), SLOT(signuid()));
+	signMailUid = actionCollection()->addAction("key_sign_mail_uid");
+	signMailUid->setIcon(KIcon("document-sign-key"));
+	connect(signMailUid, SIGNAL(triggered(bool)), SLOT(caff()));
 	importSignatureKey = actionCollection()->addAction("key_importsign");
 	importSignatureKey->setIcon(KIcon("document-import-key"));
 	connect(importSignatureKey, SIGNAL(triggered(bool)), SLOT(preimportsignkey()));
@@ -365,6 +368,7 @@ KeysManager::KeysManager(QWidget *parent)
 	m_popuppub->addAction(deleteKey);
 	m_popuppub->addAction(signKey);
 	m_popuppub->addAction(signUid);
+	m_popuppub->addAction(signMailUid);
 	m_popuppub->addAction(infoKey);
 	m_popuppub->addAction(editKey);
 	m_popuppub->addAction(refreshKey);
@@ -377,6 +381,7 @@ KeysManager::KeysManager(QWidget *parent)
 	m_popupsec->addAction(exportPublicKey);
 	m_popupsec->addAction(signKey);
 	m_popupsec->addAction(signUid);
+	m_popupsec->addAction(signMailUid);
 	m_popupsec->addAction(infoKey);
 	m_popupsec->addAction(editKey);
 	m_popupsec->addAction(refreshKey);
@@ -405,10 +410,12 @@ KeysManager::KeysManager(QWidget *parent)
 	m_popupphoto = new KMenu(this);
 	m_popupphoto->addAction(openPhoto);
 	m_popupphoto->addAction(signUid);
+	m_popupphoto->addAction(signMailUid);
 	m_popupphoto->addAction(deletePhoto);
 
 	m_popupuid = new KMenu(this);
 	m_popupuid->addAction(signUid);
+	m_popupuid->addAction(signMailUid);
 	m_popupuid->addAction(delUid);
 	m_popupuid->addAction(setPrimUid);
 
@@ -423,6 +430,7 @@ KeysManager::KeysManager(QWidget *parent)
 	editKey->setEnabled(false);
 	signKey->setEnabled(false);
 	signUid->setEnabled(false);
+	signMailUid->setEnabled(false);
 	refreshKey->setEnabled(false);
 	exportPublicKey->setEnabled(false);
 	newContact->setEnabled(false);
@@ -512,6 +520,7 @@ void KeysManager::slotGenerateKey()
 
 			KIO::getJobTracker()->registerJob(m_genkey);
 			m_genkey->start();
+			QApplication::setOverrideCursor(Qt::BusyCursor);
 		} else {
 			KConfigGroup config(KGlobal::config(), "General");
 
@@ -565,6 +574,7 @@ void KeysManager::updateStatusCounter()
 void KeysManager::slotGenerateKeyDone(KJob *job)
 {
 	changeMessage(i18nc("Application ready for user input", "Ready"));
+	QApplication::restoreOverrideCursor();
 
 	KGpgTransactionJob *tjob = qobject_cast<KGpgTransactionJob *>(job);
 
@@ -629,13 +639,20 @@ void KeysManager::slotGenerateKeyDone(KJob *job)
 
 		iview->selectNode(knode);
 
-		if (page->CBsave->isChecked()) {
-			slotrevoke(fingerprint, page->kURLRequester1->url().path(), 0, i18n("backup copy"));
+		if (page->CBsave->isChecked() || page->CBprint->isChecked()) {
+			KUrl revurl;
+			if (page->CBsave->isChecked())
+				revurl = page->kURLRequester1->url();
+
+			KGpgGenerateRevoke *genRev = new KGpgGenerateRevoke(this, fingerprint, revurl,
+					0, i18n("backup copy"));
+
+			connect(genRev, SIGNAL(done(int)), SLOT(slotRevokeGenerated(int)));
+
 			if (page->CBprint->isChecked())
-				connect(revKeyProcess, SIGNAL(revokeurl(KUrl)), SLOT(doFilePrint(KUrl)));
-		} else if (page->CBprint->isChecked()) {
-			slotrevoke(fingerprint, QString(), 0, i18n("backup copy"));
-			connect(revKeyProcess, SIGNAL(revokecertificate(QString)), this, SLOT(doPrint(QString)));
+				connect(genRev, SIGNAL(revokeCertificate(QString)), SLOT(doPrint(QString)));
+
+			genRev->start();
 		}
 		break;
 	}
@@ -775,7 +792,7 @@ void KeysManager::slotKeyRefreshDone(int result)
 	}
 
 	const QStringList log(t->getLog());
-	const QStringList keys(KGpgImport::getDetailedImportMessage(log));
+	const QStringList keys(KGpgImport::getImportedIds(log));
 	const QStringList message(KGpgImport::getImportMessage(log));
 
 	t->deleteLater();
@@ -784,7 +801,8 @@ void KeysManager::slotKeyRefreshDone(int result)
 		imodel->refreshKeys(keys);
 
 	QApplication::restoreOverrideCursor();
-	(void) new KgpgDetailedInfo(this, message.join(QString('\n')), log.join(QString('\n')), keys);
+	(void) new KgpgDetailedInfo(this, message.join(QString('\n')), log.join(QString('\n')),
+			KGpgImport::getDetailedImportMessage(log).split('\n'));
 }
 
 void KeysManager::slotDelUid()
@@ -902,8 +920,8 @@ void KeysManager::slotAddUidEnable(const QString & name)
 void KeysManager::slotAddPhoto()
 {
 	QString mess = i18n("The image must be a JPEG file. Remember that the image is stored within your public key, so "
-	"if you use a very large picture, your key will become very large as well. An image size of "
-	"around 240x288 is a good size to use.");
+	"if you use a very large picture, your key will become very large as well. The size should not exceed 6 KiB. "
+	"An image size of around 240x288 is a good size to use.");
 
 	if (KMessageBox::warningContinueCancel(0, mess) != KMessageBox::Continue)
 		return;
@@ -1253,7 +1271,7 @@ void KeysManager::showOptions()
 	if (KConfigDialog::showDialog("settings"))
 		return;
 
-	QPointer<kgpgOptions> optionsDialog = new kgpgOptions(this, "settings");
+	QPointer<kgpgOptions> optionsDialog = new kgpgOptions(this, imodel);
 	connect(optionsDialog, SIGNAL(settingsUpdated()), SLOT(readAllOptions()));
 	connect(optionsDialog, SIGNAL(homeChanged()), imodel, SLOT(refreshKeys()));
 	connect(optionsDialog, SIGNAL(refreshTrust(KgpgCore::KgpgKeyTrust, QColor)), imodel, SLOT(refreshTrust(KgpgCore::KgpgKeyTrust, QColor)));
@@ -1279,6 +1297,12 @@ void KeysManager::slotSetDefaultKey(const QString &newID)
 {
 	KGpgKeyNode *ndef = imodel->getRootNode()->findKey(newID);
 
+	if (ndef == NULL) {
+		KGpgSettings::setDefaultKey(newID);
+		KGpgSettings::self()->writeConfig();
+		return;
+	}
+
 	setDefaultKeyNode(ndef);
 }
 
@@ -1299,6 +1323,7 @@ void
 KeysManager::setActionDescriptions(int cnt)
 {
 	signUid->setText(i18np("&Sign User ID ...", "&Sign User IDs ...", cnt));
+	signMailUid->setText(i18np("Sign and &Mail User ID ...", "Sign and &Mail User IDs ...", cnt));
 	exportPublicKey->setText(i18np("E&xport Public Key...", "E&xport Public Keys...", cnt));
 	refreshKey->setText(i18np("&Refresh Key From Keyserver", "&Refresh Keys From Keyserver", cnt));
 	createGroup->setText(i18np("&Create Group with Selected Key...", "&Create Group with Selected Keys...", cnt));
@@ -1338,6 +1363,7 @@ KeysManager::slotMenu(const QPoint &pos)
 	importAllSignKeys->setEnabled(unksig && m_online);
 
 	signUid->setEnabled(!(itype & ~(ITYPE_PAIR | ITYPE_UID | ITYPE_UAT)));
+	signMailUid->setEnabled(signUid->isEnabled());
 	setActionDescriptions(cnt);
 
 	if (itype == ITYPE_SIGN) {
@@ -1384,55 +1410,48 @@ KeysManager::slotMenu(const QPoint &pos)
 	}
 }
 
-void KeysManager::slotrevoke(const QString &keyID, const KUrl &revokeUrl, const int reason, const QString &description)
-{
-	revKeyProcess = new KgpgInterface();
-	revKeyProcess->KgpgRevokeKey(keyID, revokeUrl, reason, description);
-	// FIXME: slot? delete?
-}
-
 void KeysManager::revokeWidget()
 {
-	KDialog *keyRevokeWidget = new KDialog(this);
 	KGpgNode *nd = iview->selectedNode();
+	KDialog *keyRevokeDialog = new KGpgRevokeDialog(this, nd->toKeyNode());
 
-	keyRevokeWidget->setCaption(i18n("Create Revocation Certificate"));
-	keyRevokeWidget->setButtons(KDialog::Ok | KDialog::Cancel);
-	keyRevokeWidget->setDefaultButton(KDialog::Ok);
-	keyRevokeWidget->setModal(true);
-	KgpgRevokeWidget *keyRevoke = new KgpgRevokeWidget();
+	connect(keyRevokeDialog, SIGNAL(finished(int)), SLOT(slotRevokeDialogFinished(int)));
 
-	keyRevoke->keyID->setText(i18nc("<Name> (<Email>) ID: <KeyId>", "%1 (%2) ID: %3", nd->getName(), nd->getEmail(), nd->getId()));
-	keyRevoke->kURLRequester1->setUrl(QDir::homePath() + '/' + nd->getEmail().section('@', 0, 0) + ".revoke");
-	keyRevoke->kURLRequester1->setMode(KFile::File);
-
-	keyRevoke->setMinimumSize(keyRevoke->sizeHint());
-	keyRevoke->show();
-	keyRevokeWidget->setMainWidget(keyRevoke);
-
-	if (keyRevokeWidget->exec() != QDialog::Accepted)
-		return;
-
-	if (keyRevoke->cbSave->isChecked()) {
-		slotrevoke(nd->getId(), keyRevoke->kURLRequester1->url().path(), keyRevoke->comboBox1->currentIndex(), keyRevoke->textDescription->toPlainText());
-		if (keyRevoke->cbPrint->isChecked())
-			connect(revKeyProcess, SIGNAL(revokeurl(const KUrl &)), SLOT(doFilePrint(const KUrl &)));
-		if (keyRevoke->cbImport->isChecked())
-			connect(revKeyProcess, SIGNAL(revokeurl(const KUrl &)), SLOT(slotImportRevoke(const KUrl &)));
-	} else {
-		slotrevoke(nd->getId(), QString(), keyRevoke->comboBox1->currentIndex(), keyRevoke->textDescription->toPlainText());
-		if (keyRevoke->cbPrint->isChecked())
-			connect(revKeyProcess, SIGNAL(revokecertificate(const QString &)), SLOT(doPrint(const QString &)));
-		if (keyRevoke->cbImport->isChecked())
-			connect(revKeyProcess, SIGNAL(revokecertificate(const QString &)), SLOT(slotImportRevokeTxt(const QString &)));
-	}
+	keyRevokeDialog->open();
 }
 
-void KeysManager::slotImportRevoke(const KUrl &url)
+void KeysManager::slotRevokeDialogFinished(int result)
 {
-	KGpgImport *import = new KGpgImport(this, KUrl::List(url));
-	connect(import, SIGNAL(done(int)), SLOT(slotImportDone(int)));
-	import->start();
+	sender()->deleteLater();
+
+	if (result != QDialog::Accepted)
+		return;
+
+	KGpgRevokeDialog *keyRevokeDialog = qobject_cast<KGpgRevokeDialog *>(sender());
+
+	KGpgGenerateRevoke *genRev = new KGpgGenerateRevoke(this, keyRevokeDialog->getId(), keyRevokeDialog->saveUrl(),
+			keyRevokeDialog->getReason(), keyRevokeDialog->getDescription());
+
+	connect(genRev, SIGNAL(done(int)), SLOT(slotRevokeGenerated(int)));
+
+	if (keyRevokeDialog->printChecked())
+		connect(genRev, SIGNAL(revokeCertificate(QString)), SLOT(doPrint(QString)));
+	if (keyRevokeDialog->importChecked())
+		connect(genRev, SIGNAL(revokeCertificate(QString)), SLOT(slotImportRevokeTxt(QString)));
+
+	genRev->start();
+}
+
+void KeysManager::slotRevokeGenerated(int result)
+{
+	KGpgGenerateRevoke *genRev = qobject_cast<KGpgGenerateRevoke *>(sender());
+
+	genRev->deleteLater();
+
+	if (result != KGpgTransaction::TS_OK) {
+		KMessageBox::detailedSorry(this, i18n("Creation of the revocation certificate failed..."), genRev->getOutput());
+		return;
+	}
 }
 
 void KeysManager::slotImportRevokeTxt(const QString &revokeText)
@@ -1633,16 +1652,12 @@ void KeysManager::slotProcessExportClip(int result)
 
 void KeysManager::showKeyInfo(const QString &keyID)
 {
-	KgpgInterface *iface = new KgpgInterface;
-	KgpgKeyList listkeys = iface->readJoinedKeys(TRUST_MINIMUM, QStringList(keyID));
-	delete iface;
+	KGpgKeyNode *key = imodel->getRootNode()->findKey(keyID);
 
-	if (listkeys.count() == 0)
+	if (key == NULL)
 		return;
 
-	KgpgKey *k = new KgpgKey(listkeys.at(0));
-	KgpgKeyInfo *opts = new KgpgKeyInfo(k, this);
-	opts->show();
+	showProperties(key);
 }
 
 void KeysManager::slotShowPhoto()
@@ -1857,6 +1872,10 @@ void KeysManager::editGroup()
 
 void KeysManager::signkey()
 {
+	// another sign operation is still running
+	if (!signList.isEmpty())
+		return;
+
 	KgpgItemType tp;
 	QList<KGpgNode *> tmplist = iview->selectedNodes(NULL, &tp);
 	if (tmplist.isEmpty())
@@ -1905,6 +1924,7 @@ void KeysManager::signkey()
 	QPointer<KgpgSelectSecretKey> opts = new KgpgSelectSecretKey(this, imodel, signList.count());
 	if (opts->exec() != QDialog::Accepted) {
 		delete opts;
+		signList.clear();
 		return;
 	}
 
@@ -1927,8 +1947,12 @@ void KeysManager::signkey()
 
 void KeysManager::signuid()
 {
+	// another sign operation is still running
+	if (!signList.isEmpty())
+		return;
+
 	KgpgItemType tp;
-	QList<KGpgNode *> tmplist = iview->selectedNodes(NULL, &tp);
+	KGpgNode::List tmplist = iview->selectedNodes(NULL, &tp);
 	if (tmplist.isEmpty())
 		return;
 
@@ -1984,6 +2008,7 @@ void KeysManager::signuid()
 	QPointer<KgpgSelectSecretKey> opts = new KgpgSelectSecretKey(this, imodel, signList.count());
 	if (opts->exec() != QDialog::Accepted) {
 		delete opts;
+		signList.clear();
 		return;
 	}
 
@@ -2033,7 +2058,7 @@ void KeysManager::signatureResult(int success)
 	} else {
 		ta = static_cast<KGpgSignTransactionHelper *>(static_cast<KGpgSignKey *>(sender()));
 	}
-	KGpgKeyNode *nd = ta->getKey();
+	KGpgKeyNode *nd = const_cast<KGpgKeyNode *>(ta->getKey());
 	const bool localsign = ta->getLocal();
 	const int checklevel = ta->getChecking();
 	const QString signer(ta->getSigner());
@@ -2053,13 +2078,11 @@ void KeysManager::signatureResult(int success)
 				nd->getName(), nd->getEmail()));
 		break;
 	default:
-		QPointer<KgpgDetailedConsole> q = new KgpgDetailedConsole(this,
+		if (KMessageBox::questionYesNo(this,
 				i18n("<qt>Signing key <b>%1</b> with key <b>%2</b> failed.<br />"
 						"Do you want to try signing the key in console mode?</qt>",
-						nd->getId(), signer));
-		if (q->exec() == QDialog::Accepted)
+						nd->getId(), signer)) == KMessageBox::Yes)
 			signKeyOpenConsole(signer, nd->getId(), checklevel, localsign);
-		delete q;
 	}
 
 	if (++keyCount == signList.count()) {
@@ -2071,6 +2094,53 @@ void KeysManager::signatureResult(int success)
 	}
 }
 
+void KeysManager::caff()
+{
+	KgpgItemType tp;
+	KGpgNode::List tmplist = iview->selectedNodes(NULL, &tp);
+	KGpgSignableNode::List slist;
+	if (tmplist.isEmpty())
+		return;
+
+	if (tp & ~(ITYPE_PAIR | ITYPE_UID | ITYPE_UAT)) {
+		KMessageBox::sorry(this, i18n("You can only sign user ids and photo ids. Please check your selection."));
+		return;
+	}
+
+	foreach (KGpgNode *nd, tmplist) {
+		switch (nd->getType()) {
+		case KgpgCore::ITYPE_PAIR:
+		case KgpgCore::ITYPE_PUBLIC: {
+			KGpgKeyNode *knd = qobject_cast<KGpgKeyNode *>(nd);
+			if (!knd->wasExpanded())
+				knd->getChildCount();
+			}
+		}
+		slist.append(nd->toSignableNode());
+	}
+
+	QPointer<KgpgSelectSecretKey> opts = new KgpgSelectSecretKey(this, imodel, slist.count(), false, false);
+	if (opts->exec() != QDialog::Accepted) {
+		delete opts;
+		return;
+	}
+
+	KGpgCaff *ca = new KGpgCaff(this, slist, QStringList(opts->getKeyID()), opts->getSignTrust());
+	delete opts;
+
+	connect(ca, SIGNAL(done()), SLOT(slotCaffDone()));
+	connect(ca, SIGNAL(aborted()), SLOT(slotCaffDone()));
+
+	ca->run();
+}
+
+void KeysManager::slotCaffDone()
+{
+	Q_ASSERT(qobject_cast<KGpgCaff *>(sender()) != NULL);
+
+	sender()->deleteLater();
+}
+
 void KeysManager::signKeyOpenConsole(const QString &signer, const QString &keyid, const int checking, const bool local)
 {
 	KConfigGroup config(KGlobal::config(), "General");
@@ -2176,6 +2246,11 @@ void KeysManager::importRemoteFinished(int result)
 		imodel->refreshKeys(keys);
 }
 
+void KeysManager::refreshKeys(const QStringList& ids)
+{
+	imodel->refreshKeys(ids);
+}
+
 void KeysManager::delsignkey()
 {
 	KGpgNode *nd = iview->selectedNode();
@@ -2220,21 +2295,24 @@ void KeysManager::delsignkey()
 	if (KMessageBox::questionYesNo(this, ask, QString(), KStandardGuiItem::del(), KStandardGuiItem::cancel()) != KMessageBox::Yes)
 		return;
 
-	KgpgInterface *delSignKeyProcess = new KgpgInterface();
-	connect(delSignKeyProcess, SIGNAL(delsigfinished(bool)), this, SLOT(delsignatureResult(bool)));
-	delSignKeyProcess->KgpgDelSignature(parentKey, uid, signID);
+	KGpgDelSign *delsig = new KGpgDelSign(this, nd->toSignNode());
+	connect(delsig, SIGNAL(done(int)), SLOT(delsignatureResult(int)));
+	delsig->start();
 }
 
-void KeysManager::delsignatureResult(bool success)
+void KeysManager::delsignatureResult(int success)
 {
-	if (success) {
+	sender()->deleteLater();
+
+	if (success == KGpgTransaction::TS_OK) {
 		KGpgNode *nd = iview->selectedNode()->getParentKeyNode();
 
 		while (!(nd->getType() & ITYPE_PAIR))
 			nd = nd->getParentKeyNode();
 		imodel->refreshKey(nd->toKeyNode());
-	} else
+	} else {
 		KMessageBox::sorry(this, i18n("Requested operation was unsuccessful, please edit the key manually."));
+	}
 }
 
 void KeysManager::slotedit()
@@ -2269,18 +2347,6 @@ void KeysManager::slotEditDone(int exitcode)
 	editKey->setEnabled(true);
 }
 
-void KeysManager::doFilePrint(const KUrl &url)
-{
-	QFile qfile(url.toLocalFile());
-
-	if (qfile.open(QIODevice::ReadOnly)) {
-		QTextStream t(&qfile);
-		doPrint(t.readAll());
-	} else {
-		KMessageBox::sorry(this, i18n("<qt>Cannot open file <b>%1</b> for printing...</qt>", url.toLocalFile()));
-	}
-}
-
 void KeysManager::doPrint(const QString &txt)
 {
 	QPrinter prt;
@@ -2477,7 +2543,7 @@ void KeysManager::slotDelKeyDone(int res)
 void KeysManager::slotPreImportKey()
 {
 	QPointer<KDialog> dial = new KDialog(this);
-	dial->setCaption(i18n("Key Import") );
+	dial->setCaption(i18n("Key Import"));
 	dial->setButtons(KDialog::Ok | KDialog::Cancel);
 	dial->setDefaultButton(KDialog::Ok);
 	dial->setModal(true);
@@ -2499,8 +2565,8 @@ void KeysManager::slotPreImportKey()
 		} else {
 			slotImport(kapp->clipboard()->text(m_clipboardmode));
 		}
-
 	}
+
 	delete dial;
 }
 
@@ -2509,12 +2575,27 @@ void KeysManager::slotImport(const QString &text)
 	if (text.isEmpty())
 		return;
 
-	startImport(new KGpgImport(this, text));
+	KGpgImport *imp;
+
+	if (!KGpgImport::isKey(text) && KGpgDecrypt::isEncryptedText(text)) {
+		if (KMessageBox::questionYesNo(this,
+				i18n("<qt>The text in the clipboard does not look like a key, but like encrypted text.<br />Do you want to decrypt it first and then try importing it?</qt>"),
+					       i18n("Import from Clipboard")) != KMessageBox::Yes)
+			return;
+
+		imp = new KGpgImport(this);
+		KGpgDecrypt *decr = new KGpgDecrypt(this, text);
+		imp->setInputTransaction(decr);
+	} else {
+		imp = new KGpgImport(this, text);
+	}
+
+	startImport(imp);
 }
 
 void KeysManager::slotImport(const KUrl::List &files)
 {
-	startImport(new KGpgImport(this, KUrl::List(files)));
+	startImport(new KGpgImport(this, files));
 }
 
 void KeysManager::startImport(KGpgImport *import)
@@ -2527,30 +2608,32 @@ void KeysManager::startImport(KGpgImport *import)
 void KeysManager::slotImportDone(int result)
 {
 	KGpgImport *import = qobject_cast<KGpgImport *>(sender());
-
-	slotImportDone(import, result);
-}
-
-void KeysManager::slotImportDone(KGpgImport *import, int result)
-{
+	Q_ASSERT(import != NULL);
 	const QStringList rawmsgs(import->getMessages());
 
 	if (result != 0) {
-		KMessageBox::errorList(this, i18n("Key importing failed. Please see the detailed log for more information."), rawmsgs, i18n("Key Import"));
-		import->deleteLater();
+		KMessageBox::detailedSorry(this, i18n("Key importing failed. Please see the detailed log for more information."),
+				rawmsgs.join("\n"), i18n("Key Import"));
 	}
 
-	const QStringList keys(import->getImportedIds(0x1f));
-	const QString msg(import->getImportMessage());
-	const QStringList keynames(import->getImportedKeys());
-	import->deleteLater();
+	QStringList keys(import->getImportedIds(0x1f));
+	const bool needsRefresh = !keys.isEmpty();
+	keys << import->getImportedIds(0);
 
-	new KgpgDetailedInfo(this, msg, rawmsgs.join("\n"), keynames);
+	if (!keys.isEmpty()) {
+		const QString msg(import->getImportMessage());
+		const QStringList keynames(import->getImportedKeys());
 
-	if (!keys.isEmpty())
-		imodel->refreshKeys(keys);
-	else
+		new KgpgDetailedInfo(this, msg, rawmsgs.join("\n"), keynames, i18n("Key Import"));
+		if (needsRefresh)
+			imodel->refreshKeys(keys);
+		else
+			changeMessage(i18nc("Application ready for user input", "Ready"));
+	} else{
 		changeMessage(i18nc("Application ready for user input", "Ready"));
+	}
+
+	import->deleteLater();
 }
 
 void KeysManager::refreshkey()
@@ -2564,14 +2647,6 @@ KGpgItemModel *KeysManager::getModel()
 	return imodel;
 }
 
-void
-KeysManager::slotRefreshKeys(KgpgInterface *iface, const QStringList &keys)
-{
-	iface->deleteLater();
-	if (!keys.isEmpty())
-		imodel->refreshKeys(keys);
-}
-
 void
 KeysManager::slotNetworkUp()
 {
@@ -2693,7 +2768,7 @@ KeysManager::clipEncrypt()
 			selec = dialog->selectedKeys();
 
 		KGpgTextInterface *txtEncrypt = new KGpgTextInterface();
-		connect (txtEncrypt, SIGNAL(txtEncryptionFinished(QString, KGpgTextInterface *)), SLOT(slotSetClip(QString, KGpgTextInterface *)));
+		connect (txtEncrypt, SIGNAL(txtEncryptionFinished(QString)), SLOT(slotSetClip(QString)));
 		m_trayicon->setStatus(KStatusNotifierItem::Active);
 		txtEncrypt->encryptText(cliptext, selec, options);
 	}
@@ -2702,9 +2777,9 @@ KeysManager::clipEncrypt()
 }
 
 void
-KeysManager::slotSetClip(const QString &newtxt, KGpgTextInterface *iface)
+KeysManager::slotSetClip(const QString &newtxt)
 {
-	iface->deleteLater();
+	sender()->deleteLater();
 
 	m_trayicon->setStatus(KStatusNotifierItem::Passive);
 
