@@ -27,8 +27,8 @@
 #include <QString>
 
 class KPassivePopup;
-class KgpgInterface;
 class KGpgTextInterface;
+class KGpgItemModel;
 
 class KgpgLibrary : public QObject
 {
@@ -38,34 +38,33 @@ public:
     /**
      * Initialize the class
      */
-    explicit KgpgLibrary(QWidget *parent = 0, const bool &pgpExtension = false);
+    explicit KgpgLibrary(QWidget *parent = 0);
+
+    /**
+     * set the extension used for the encrypted file (including the '.')
+     */
+    void setFileExtension(const QString &extension);
 
 signals:
     void encryptionOver();
-    void decryptionOver(KgpgLibrary *, KUrl);
-    void importOver(QStringList);
+    void decryptionOver(KUrl);
+    void importOver(KgpgLibrary *, QStringList);
     void systemMessage(QString, bool reset = false);
     void photoAdded();
 
 public slots:
-    void slotFileEnc(const KUrl::List &urls = KUrl(""), const QStringList &opts = QStringList(), const QStringList &defaultKey = QStringList(), const KShortcut &goDefaultKey = KShortcut(QKeySequence(Qt::CTRL + Qt::Key_Home)));
+    void slotFileEnc(const KUrl::List &urls, const QStringList &opts, KGpgItemModel *model, const KShortcut &goDefaultKey, const QString &defaultKey = QString());
     void slotFileDec(const KUrl &srcUrl, const KUrl &destUrl, const QStringList &customDecryptOption = QStringList());
 
-    void addPhoto(const QString &keyid);
-
 private slots:
-    void startEncode(const QStringList &encryptkeys, const QStringList &encryptoptions, const bool &symetric);
-    void fastEncode(const KUrl &filetocrypt, const QStringList &encryptkeys, const QStringList &encryptoptions, const bool &symetric);
-    void processEnc(const KUrl &, KGpgTextInterface*);
-    void processEncError(const QString &mssge, KGpgTextInterface*);
-    void processDecOver(int, KGpgTextInterface*);
-    void processDecError(const QString &mssge);
+    void processEnc(const KUrl &);
+    void processEncError(const QString &mssge);
+    void processDecOver(int);
     void processEncPopup(const KUrl &url);
-    void processPopup2(const QString &fileName);
-
-    void slotAddPhotoFinished(int res, KgpgInterface *interface);
 
 private:
+    void startEncode(const QStringList &encryptkeys, const QStringList &encryptoptions, const bool symetric);
+    void fastEncode(const KUrl &filetocrypt, const QStringList &encryptkeys, const QStringList &encryptoptions, const bool symetric);
     QWidget *m_panel;
     QString m_extension;
     QStringList m_encryptkeys;
