@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2002 Jean-Baptiste Mardelle <bj@altern.org>
  * Copyright (C) 2007,2008 Rolf Eike Beer <kde@opensource.sf-tec.de>
  */
@@ -27,26 +27,38 @@
 #include <KHBox>
 #include <KVBox>
 
-#include "kgpginterface.h"
-#include "kgpgsettings.h"
 #include "images.h"
-#include "selectkeyproxymodel.h"
 #include "kgpgitemmodel.h"
+#include "KGpgRootNode.h"
+#include "kgpgsettings.h"
+#include "selectkeyproxymodel.h"
 
 
 using namespace KgpgCore;
 
-KgpgSelectPublicKeyDlg::KgpgSelectPublicKeyDlg(QWidget *parent, KGpgItemModel *model, const KShortcut &goDefaultKey, const QString &sfile, const bool &hideasciioption)
-                      : KDialog(parent)
+KgpgSelectPublicKeyDlg::KgpgSelectPublicKeyDlg(QWidget *parent, KGpgItemModel *model, const KShortcut &goDefaultKey, const bool hideasciioption, const KUrl::List &files)
+	: KDialog(parent),
+	m_customoptions(NULL),
+	imodel(model),
+	m_hideasciioption(hideasciioption)
 {
-    setCaption(i18n("Select Public Key"));
     setButtons(Details | Ok | Cancel);
     setDefaultButton(Ok);
     setButtonText(Details, i18n("O&ptions"));
 
-    m_fmode = (sfile != NULL);
-    if (m_fmode)
-        setCaption(i18n("Select Public Key for %1", sfile));
+    int fcount = files.count();
+    m_fmode = (fcount > 0);
+
+    switch (fcount) {
+    case 0:
+	setCaption(i18n("Select Public Key"));
+	break;
+    case 1:
+	setCaption(i18n("Select Public Key for %1", files.first().fileName()));
+	break;
+    default:
+	setCaption(i18np("Select Public Key for %2 and one more file", "Select Public Key for %2 and %1 more files", files.count() - 1, files.first().fileName()));
+    }
 
     QWidget *page = new QWidget(this);
 
@@ -60,8 +72,6 @@ KgpgSelectPublicKeyDlg::KgpgSelectPublicKeyDlg(QWidget *parent, KGpgItemModel *m
     m_searchlineedit->setClearButtonShown(true);
     searchlabel->setBuddy(m_searchlineedit);
 
-    imodel = model;
-
     iproxy = new SelectKeyProxyModel(this);
     iproxy->setKeyModel(imodel);
 
@@ -78,7 +88,6 @@ KgpgSelectPublicKeyDlg::KgpgSelectPublicKeyDlg(QWidget *parent, KGpgItemModel *m
     optionsbox->setFrameShape(QFrame::StyledPanel);
     setDetailsWidget(optionsbox);
 
-    m_hideasciioption = hideasciioption;
     if (m_hideasciioption)
         m_cbarmor = 0;
     else
@@ -111,7 +120,6 @@ KgpgSelectPublicKeyDlg::KgpgSelectPublicKeyDlg(QWidget *parent, KGpgItemModel *m
     dialoglayout->addWidget(m_keyslist);
     page->setLayout(dialoglayout);
 
-    m_customoptions = 0;
     if (KGpgSettings::allowCustomEncryptionOptions())
     {
         KHBox *expertbox = new KHBox(page);
@@ -147,23 +155,22 @@ KgpgSelectPublicKeyDlg::KgpgSelectPublicKeyDlg(QWidget *parent, KGpgItemModel *m
 
 QStringList KgpgSelectPublicKeyDlg::selectedKeys() const
 {
-    if (getSymmetric())
-        return QStringList();
+	if (getSymmetric())
+		return QStringList();
 
-    QStringList selectedKeys;
+	QStringList selectedKeys;
 
-	QModelIndexList sel = m_keyslist->selectionModel()->selectedIndexes();
-	for (int i = 0; i < sel.count(); i++) {
-		if (sel.at(i).column() != 0)
+	foreach (const QModelIndex &idx, m_keyslist->selectionModel()->selectedIndexes()) {
+		if (idx.column() != 0)
 			continue;
-		KGpgNode *nd = iproxy->nodeForIndex(sel.at(i));
+		KGpgNode *nd = iproxy->nodeForIndex(idx);
 		if (nd->getType() == ITYPE_GROUP)
 			selectedKeys << nd->getName();
 		else
 			selectedKeys << nd->getId();
 	}
 
-    return selectedKeys;
+	return selectedKeys;
 }
 
 bool KgpgSelectPublicKeyDlg::getSymmetric() const
