@@ -40,10 +40,10 @@ import sys
 import time
 
 from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from PyQt4.QtGui import QWidget, QKeySequence, QHeaderView, QTreeWidgetItem, QCursor
 from PyQt4 import uic
-from PyKDE4.kdecore import i18n, i18nc, i18np, i18ncp, ki18n, KAboutData, KCmdLineArgs, KCmdLineOptions, KStandardDirs, KLocalizedString
-from PyKDE4.kdeui import KApplication, KXmlGuiWindow, KStandardAction, KIcon, KToggleAction, KNotification, KMessageBox
+from PyKDE4.kdecore import i18n, i18nc, i18np, i18ncp, ki18n, KAboutData, KCmdLineArgs, KCmdLineOptions, KStandardDirs, KLocalizedString, KToolInvocation
+from PyKDE4.kdeui import KApplication, KXmlGuiWindow, KStandardAction, KIcon, KAction, KToggleAction, KNotification, KMenu, KMessageBox, KStatusNotifierItem
 
 def translate(self, prop):
     """reimplement method from uic to change it to use gettext"""
@@ -182,11 +182,11 @@ def collect_printer_state_reasons (connection):
         for reason in reasons:
             if reason == "none":
                 break
-            if (reason.startswith ("moving-to-paused") or
-                reason.startswith ("paused") or
-                reason.startswith ("shutdown") or
-                reason.startswith ("stopping") or
-                reason.startswith ("stopped-partly")):
+            if (reason.startswith(QLatin1String("moving-to-paused")) or
+                reason.startswith(QLatin1String("paused")) or
+                reason.startswith(QLatin1String("shutdown")) or
+                reason.startswith(QLatin1String("stopping")) or
+                reason.startswith(QLatin1String("stopped-partly"))):
                 continue
             result.append (StateReason (name, reason))
     return result
@@ -234,31 +234,27 @@ class JobManager(QObject, monitor.Watcher):
             APPDIR = QDir.currentPath()
         else:
             file =  KStandardDirs.locate("appdata", "printer-applet.ui")
-            APPDIR = file.left(file.lastIndexOf("/"))
+            APPDIR = file.left(file.lastIndexOf('/'))
 
         self.mainWindow = MainWindow()
-        uic.loadUi(APPDIR + "/" + "printer-applet.ui", self.mainWindow)
+        uic.loadUi(unicode(APPDIR + '/' + "printer-applet.ui"), self.mainWindow)
 
         self.printersWindow = PrintersWindow(self)
-        uic.loadUi(APPDIR + "/" + "printer-applet-printers.ui", self.printersWindow)
+        uic.loadUi(unicode(APPDIR + '/' + "printer-applet-printers.ui"), self.printersWindow)
 
-        self.sysTray = QSystemTrayIcon(KIcon("printer"), self.mainWindow)
-        #self.sysTray.show()
-        self.connect(self.sysTray, SIGNAL("activated( QSystemTrayIcon::ActivationReason )"), self.toggle_window_display)
-
-        self.menu = QMenu()
-        self.menu.addAction(i18n("_Hide").replace("_", ""), self.on_icon_hide_activate)
-        self.menu.addAction(KIcon("application-exit"), i18n("Quit"), self.on_icon_quit_activate)
-        self.sysTray.setContextMenu(self.menu)
+        self.sysTray = KStatusNotifierItem(self.mainWindow)
+        self.sysTray.setCategory(KStatusNotifierItem.Hardware)
+        self.sysTray.setIconByName("printer")
+        self.sysTray.setAssociatedWidget(self.mainWindow)
 
         self.mainWindow.treeWidget.setContextMenuPolicy(Qt.CustomContextMenu)
         self.connect(self.mainWindow.treeWidget, SIGNAL("customContextMenuRequested(const QPoint&)"), self.show_treeview_popup_menu)
         #self.connect(self.mainWindow.treeWidget, SIGNAL("itemClicked(QTreeWidgetItem*, int)"), self.printItemClicked)
-        self.rightClickMenu = QMenu(self.mainWindow.treeWidget)
+        self.rightClickMenu = KMenu(self.mainWindow.treeWidget)
         self.cancel = self.rightClickMenu.addAction(i18n("Cancel"), self.on_job_cancel_activate)
-        self.hold = self.rightClickMenu.addAction(i18n("_Hold").replace("_",""), self.on_job_hold_activate)
-        self.release = self.rightClickMenu.addAction(i18n("_Release").replace("_",""), self.on_job_release_activate)
-        self.reprint = self.rightClickMenu.addAction(i18n("Re_print").replace("_",""), self.on_job_reprint_activate)
+        self.hold = self.rightClickMenu.addAction(i18n("_Hold").remove('_'), self.on_job_hold_activate)
+        self.release = self.rightClickMenu.addAction(i18n("_Release").remove('_'), self.on_job_release_activate)
+        self.reprint = self.rightClickMenu.addAction(i18n("Re_print").remove('_'), self.on_job_reprint_activate)
 
         closeAction = KStandardAction.close(self.hideMainWindow, self.mainWindow.actionCollection());
 
@@ -268,14 +264,20 @@ class JobManager(QObject, monitor.Watcher):
         refreshAction.setShortcut(QKeySequence(Qt.Key_F5))
         self.connect(refreshAction, SIGNAL("triggered(bool)"), self.on_refresh_activate);
 
-        showCompletedJobsAction = KToggleAction( i18n( "Show Completed Jobs" ), self.mainWindow)
+        configureAction = KAction("Printer Configuration", self.mainWindow)
+        configureAction.setText(i18n("Printer Configuration..."))
+        configureAction.setIcon(KIcon("configure"))
+        self.mainWindow.actionCollection().addAction("printer_configuration", configureAction)
+        self.connect(configureAction, SIGNAL("triggered(bool)"), self.on_printer_configuration_activate)
+
+        showCompletedJobsAction = KToggleAction( i18n( "Show Completed Jobs"), self.mainWindow)
         self.mainWindow.actionCollection().addAction("show_completed_jobs", showCompletedJobsAction)
         self.connect(showCompletedJobsAction, SIGNAL("triggered(bool)"), self.on_show_completed_jobs_activate);
 
-        showPrinterStatusAction = KToggleAction( i18n( "Show Printer Status" ), self.mainWindow)
+        showPrinterStatusAction = KToggleAction( i18n( "Show Printer Status"), self.mainWindow)
         self.mainWindow.actionCollection().addAction("show_printer_status", showPrinterStatusAction)
         self.connect(showPrinterStatusAction, SIGNAL("triggered(bool)"), self.on_show_printer_status_activate);
-        
+
         self.mainWindow.treeWidget.header().setResizeMode(QHeaderView.ResizeToContents)
         self.printersWindow.treeWidget.header().setResizeMode(QHeaderView.ResizeToContents)
 
@@ -316,7 +318,7 @@ class JobManager(QObject, monitor.Watcher):
     """
 
     def notify_new_printer (self, printer, title, text):
-        self.sysTray.show()
+        self.sysTray.setStatus(KStatusNotifierItem.Active)
         KNotification.event(title, text, KIcon("konqueror").pixmap(QSize(22,22)))
 
     """unused, see set_special_statusicon
@@ -355,14 +357,6 @@ class JobManager(QObject, monitor.Watcher):
         error_text = error_text.replace("span", "strong")
         KMessageBox.error(self.mainWindow, error_text, i18n("Error"))
 
-    def toggle_window_display(self, activationReason):
-        if activationReason == QSystemTrayIcon.Trigger:
-            if self.mainWindow.isVisible():
-                self.mainWindow.hide()
-            else:
-                self.mainWindow.show()
-                self.monitor.refresh()
-    
     #FIXME, hide printer status window?
     def hideMainWindow(self):
         self.mainWindow.hide()
@@ -380,6 +374,9 @@ class JobManager(QObject, monitor.Watcher):
         else:
             self.printersWindow.hide()
 
+    def on_printer_configuration_activate(self, activated):
+        KToolInvocation.startServiceByDesktopName("system-config-printer-kde")
+
     """not using notifications in qt frontend
     def on_notification_closed(self, notify):
     """
@@ -533,8 +530,10 @@ class JobManager(QObject, monitor.Watcher):
         debugprint ("num_jobs: %d" % num_jobs)
         debugprint ("num_jobs_when_hidden: %d" % self.num_jobs_when_hidden)
 
-        self.sysTray.setVisible(self.special_status_icon or
-                                     num_jobs > self.num_jobs_when_hidden)
+        if self.special_status_icon or num_jobs > self.num_jobs_when_hidden:
+            self.sysTray.setStatus(KStatusNotifierItem.Active)
+        else:
+            self.sysTray.setStatus(KStatusNotifierItem.Passive)
 
     def show_treeview_popup_menu(self, postition):
         # Right-clicked.
@@ -566,13 +565,6 @@ class JobManager(QObject, monitor.Watcher):
     def on_icon_popupmenu(self, icon, button, time):
         self.icon_popupmenu.popup (None, None, None, button, time)
 
-    def on_icon_hide_activate(self):
-        self.num_jobs_when_hidden = len (self.jobs.keys ())
-        self.set_statusicon_visibility ()
-
-    def on_icon_quit_activate(self):
-        app.quit()
-
     def on_job_cancel_activate(self):
         try:
             c = authconn.Connection (self.mainWindow)
@@ -656,7 +648,7 @@ class JobManager(QObject, monitor.Watcher):
             else:
                 tooltip = i18np("1 document queued", "%1 documents queued", num_jobs)
 
-        self.sysTray.setToolTip(tooltip)
+        self.sysTray.setToolTip("printer", i18n("Print Status"), tooltip)
 
     def update_status (self, have_jobs=None):
         # Found out which printer state reasons apply to our active jobs.
@@ -918,7 +910,7 @@ class JobManager(QObject, monitor.Watcher):
 
         (title, text) = reason.get_description ()
         printer = reason.get_printer ()
-        
+
         iter = QTreeWidgetItem(self.printersWindow.treeWidget)
         iter.setText(0, reason.get_printer ())
         iter.setText(1, printer)
@@ -1132,7 +1124,7 @@ if __name__ == "__main__":
     appName     = "printer-applet"
     catalogue   = "printer-applet"
     programName = ki18n("Printer Applet")
-    version     = "1.4"
+    version     = "1.5"
     description = ki18n("Applet to view current print jobs and configure new printers")
     license     = KAboutData.License_GPL
     copyright   = ki18n("2007-2008 Canonical Ltd")
@@ -1161,5 +1153,5 @@ if __name__ == "__main__":
     applet = JobManager()
     if args.isSet("show"):
         applet.mainWindow.show()
-        applet.sysTray.show()
+        applet.sysTray.setStatus(KStatusNotifierItem.Active)
     sys.exit(app.exec_())
